package eu.mhsl.craftattack.worldmuseum.worlds;

import com.google.gson.JsonObject;
import com.google.gson.JsonParser;
import net.kyori.adventure.text.Component;
import net.minestom.server.MinecraftServer;
import net.minestom.server.coordinate.Pos;
import net.minestom.server.instance.AnvilLoader;
import net.minestom.server.instance.InstanceContainer;
import net.minestom.server.item.ItemStack;
import net.minestom.server.item.Material;
import net.minestom.server.utils.NamespaceID;
import net.minestom.server.world.DimensionType;
import java.io.*;
import java.util.UUID;

public class World extends InstanceContainer {
    private final static DimensionType dimension = DimensionType.builder(NamespaceID.from("mhsl:worldview_by_olischma")).height(400).ambientLight(2.0f).build();
    private final ItemStack item;
    private String name = "default";
    private Pos spawn = new Pos(0,0,0);
    private boolean enabled = false;
    private final String defaultConfig = "{\n" +
            "  \"enabled\": false,\n" +
            "  \"name\": \"DEFAULT_NAME\",\n" +
            "  \"material\": \"GRASS_BLOCK\",\n" +
            "  \"spawn\": {\n" +
            "    \"x\": 0,\n" +
            "    \"y\": 0,\n" +
            "    \"z\": 0\n" +
            "  },\n" +
            "  \"tags\": [\n" +
            "    \"RANDOM_TAG\"\n" +
            "  ]\n" +
            "}";

    static {
        MinecraftServer.getDimensionTypeManager().addDimension(dimension);
    }
    AnvilLoader anvilLoader;

    public World(File file) {
        super(UUID.randomUUID(), dimension, new AnvilLoader(file.getPath()));
        this.anvilLoader = new AnvilLoader(file.getPath());
        MinecraftServer.getInstanceManager().registerInstance(this);
        try {
            File checkFile = new File(file.getPath() + "/config.json");
            if (checkFile.createNewFile()) {
                FileWriter fileWriter = new FileWriter(file.getPath() + "/config.json");
                fileWriter.write(defaultConfig);
                fileWriter.close();
            }
        } catch (IOException e){
            e.printStackTrace();
        }


        Material material = Material.GRASS_BLOCK;
        try {
            JsonObject config = (JsonObject) JsonParser.parseReader(new FileReader(file.getPath() + "/config.json"));
            this.name = config.get("name").getAsString();
            JsonObject spawn = config.getAsJsonObject("spawn");
            this.spawn = new Pos(spawn.get("x").getAsDouble(), spawn.get("y").getAsDouble(), spawn.get("z").getAsDouble());
            material = Material.fromNamespaceId("minecraft:" + config.get("material").getAsString().toLowerCase());
            this.enabled = config.get("enabled").getAsBoolean();

        } catch (FileNotFoundException e) {
            e.printStackTrace();
        }
        assert material != null;
        item = ItemStack.builder(material).displayName(Component.text(name)).build();
    }

    public ItemStack getItem() {
        return item;
    }

    public String getName() {
        return name;
    }

    public Pos getSpawn() {
        return spawn;
    }
    public boolean isEnabled() {
        return enabled;
    }
}
